
/*
 * FILE:
 * gui.c
 *
 * FUNCTION:
 * GUI elements
 *
 * HISTORY:
 * Linas Vepstas January 2002
 */

#include <Common.h>
#include <System/SysTraps.h>
#include <System/DataMgr.h>
#include <System/SystemMgr.h>
#include <System/Preferences.h>
#include <System/DateTime.h>
#include <System/ErrorMgr.h>
#include <System/StringMgr.h>
#include <System/Unix/unix_stdio.h>
#include <UI/Rect.h>
#include <UI/Font.h>
#include <UI/Window.h>
#include <UI/Event.h>
#include <UI/Control.h>
#include <UI/Field.h>
#include <UI/List.h>
#include <UI/ScrollBar.h>
#include <UI/Table.h>
#include <UI/Form.h>

#include "linas-types.h"
#include "gui-rc.h"
#include "gui.h"
#include "nmeadb.h"
#include "graph.h"
#include "tzgps.h"

// ==========================================================

typedef enum
{
   gpsWaypointName=0,
   gpsWaypointNote,
   gpsWaypointNmea,
   gpsNumFields
} gpsField;

/* The curent record will be stored on the heap, as a collection
 * of strings */
struct WaypointFields_s
{
   VoidHand field_handles [gpsNumFields];
   int lengths [gpsNumFields];
};

typedef struct WaypointFields_s WaypointFields;

WaypointFields curr_rec;

#define FSTR_CPY(i, str) {                      \
   VoidHand h = curr_rec.field_handles[i];      \
   MemPtr mp = MemHandleLock (h);               \
   char * p = (char *) mp;                      \
   StrCopy (p, str);                            \
   MemPtrUnlock(mp);                            \
}

#define EMPTY(i) {                              \
   VoidHand h = curr_rec.field_handles[i];      \
   MemPtr mp = MemHandleLock (h);               \
   char * p = (char *) mp;                      \
   p[0] = 0;                                    \
   MemPtrUnlock(mp);                            \
}

/* initialize memory on the heap for a set of strings */
Int16
InitWaypointFields (void)
{
   int i;
   VoidHand h;

   /* most fields aer 50 bytes max */
   for (i=0; i<gpsNumFields; i++)
   {
      h = MemHandleNew (50);
      curr_rec.field_handles[i] = h;
   }

   /* make these into zero-length strings */
   for (i=0; i<gpsNumFields; i++)
   {
      EMPTY(i);
   }

   return 0;
}

void
FreeWaypointFields (void)
{
   int i;

   for (i=0; i<gpsNumFields; i++)
   {
      MemHandleFree (curr_rec.field_handles[i]);
   }
}

void
ResetWaypointFields (void)
{
   /* we list most, but not all fields here */
   EMPTY (gpsWaypointName);
   EMPTY (gpsWaypointNote);
}

// ==========================================================

static int
PackedSize (void)
{
   int sz;
   int i;
   VoidHand h;
   MemPtr mp;
   char * p;

   sz = 0;
   for (i=0; i<gpsNumFields; i++)
   {
      h = curr_rec.field_handles[i];
      mp = MemHandleLock (h);
      p = (char *) mp;
      curr_rec.lengths[i] = StrLen (p) + 1;
      sz += curr_rec.lengths[i];
      MemPtrUnlock(mp);
   }

   sz ++;      // just for good luck
   return sz;
}

// ==========================================================
/* copy collection of strings to the database */

Int16
StoreWaypointRecord (void)
{
   UInt16       index = 0;
   VoidHand     RecHandle;
   MemPtr       RecPointer;
   int sz;
   int i;
   VoidHand h;
   MemPtr mp;
   char * p;

   RecHandle = DmNewRecord(WaypointDB, &index, PackedSize());
   RecPointer = MemHandleLock(RecHandle);

   sz = 0;
   for (i=0; i<gpsNumFields; i++)
   {
      h = curr_rec.field_handles[i];
      mp = MemHandleLock (h);
      p = (char *) mp;
      DmWrite(RecPointer, sz, p, curr_rec.lengths[i]);
      sz += curr_rec.lengths[i];
      MemPtrUnlock(mp);
   }

   MemPtrUnlock(RecPointer);
   DmReleaseRecord(WaypointDB, index, true);

   return index;
}


// ==========================================================
// ==========================================================
// ==========================================================
// ==========================================================

static Boolean
gpsPageFlip (Int16 butid)
{
   switch (butid)
   {
      case gupStatusPageButton:
         FrmGotoForm(gupStatusPage);
         return 1;
      case gupNewWaypointPageButton:
         FrmGotoForm(gupNewWaypointPage);
         return 1;
   }
   return 0;
}

// ==========================================================

Boolean
gpsSplashPageHandler(EventPtr event)
{
   FormPtr   form;

   switch (event->eType)
   {
      case frmOpenEvent:
         form = FrmGetActiveForm();
         FrmDrawForm(form);
         return 1;

      case ctlSelectEvent:
         return gpsPageFlip (event->data.ctlEnter.controlID);

      default:
         return 0;
   }
   return 1;
}

// ==========================================================

Boolean
gpsStatusPageHandler(EventPtr event)
{
   FormPtr   form;

   switch (event->eType)
   {
      case frmOpenEvent:
         form = FrmGetActiveForm();
         FrmDrawForm(form);
			draw_sat_plot ();
         return 1;

      case ctlSelectEvent:
         return gpsPageFlip (event->data.ctlEnter.controlID);

      default:
         return 0;
   }
   return 1;
}

// ==========================================================

Handle
GetWaypointField (gpsField i)
{
   return (Handle) curr_rec.field_handles[i];
}


#define FSET(form,NAME) {                                            \
   FieldPtr fp;                                                      \
   fp = FrmGetObjectPtr(form, FrmGetObjectIndex(form, gup##NAME));   \
   FldSetTextHandle(fp, GetWaypointField (gps##NAME));               \
}

#define FRES(form,NAME) {                                            \
   FieldPtr fp;                                                      \
   fp = FrmGetObjectPtr(form, FrmGetObjectIndex(form, gup##NAME));   \
   FldSetTextHandle(fp, NULL);                                       \
}

extern int fastmode;

Boolean
gpsNewWaypointPageHandler(EventPtr event)
{
   FormPtr   form;

   switch (event->eType)
   {
      case nilEvent: return 0;
      case frmOpenEvent:
         form = FrmGetActiveForm();

         FSET (form, WaypointName);
         FSET (form, WaypointNote);
         FrmDrawForm(form);
         return 1;

      case frmCloseEvent:
         form = FrmGetActiveForm();
         FRES (form, WaypointName);
         FRES (form, WaypointNote);
         return 1;

      case ctlSelectEvent:
         if (gupLogWaypointButton == event->data.ctlEnter.controlID)
         {

				/* quick -- grab a copy of a recent fix */
				/* hack alert -- we should not do this if the last fix is old. */
				FSTR_CPY (gpsWaypointNmea, last_fix_nmea);
            StoreWaypointRecord();
            // ResetWaypointFields();
            return 1;
         }
         return gpsPageFlip (event->data.ctlEnter.controlID);

      default:
         return 0;
   }
   return 1;
}

// =================== END OF FILE ========================
